.template-addressblock_detail_view .addressText,
.template-addressblock_detail_view .addressMap {
  width:100%;
}

.sl-text-wrapper .addressText,
.sl-text-wrapper .addressMap {
  width: 50%;
  vertical-align: top;
  float:left;
}

/*Hide overlay*/
.olLayerGooglePoweredBy.olLayerGoogleV3.gmnoprint {
  display: none;
}

.simplelayout-block-wrapper.ListingBlock table.listing {
  width:100%;
}

.simplelayout-block-wrapper.ListingBlock table caption {
  display: none;
}

.simplelayout-block-wrapper.ListingBlock .gallery {
  clear:both;
}

.simplelayout-block-wrapper.ListingBlock .gallery .box{
  margin-left: 0.5em;
  margin-bottom: 0.5em;
  float:left;
}
.simplelayout-block-wrapper.ListingBlock .gallery .box {
  overflow:hidden;
  text-align: center;
  background-color: #eee;
  border:1px solid #ddd;
}
.simplelayout-block-wrapper.ListingBlock .gallery .box .frame{
  display:table-cell;
  overflow: hidden;
  vertical-align: middle;
  background-color: #eee;
  border:1px solid #ddd;
}

/* @group news listing */
.newsListing .tileItem img {
  float: left;
  padding: 1em 1em 1em 0;
}
/* @end */

/* @group news block */

.newsPublicationDate {
  color: #666666;
  font-size: 85%;
  margin-bottom: 1em;
}

/* @end */

/* @group archive portlet */
.portletArchiveListing .highlight a {
  font-weight: bold;
}
.portletArchiveListing li.year {
  cursor: pointer;
}
.portletArchiveListing li {
  display: block;
  margin-bottom: 2px;
}
.portletArchiveListing ul.months {
  display: none;
  margin-bottom: 10px;
  margin-top: 5px;
}
.portletArchiveListing li.month {
  padding-left: 2em;
}

.portletArchiveListing li.year span.yearnumber:before {
  content: '►';
  padding-right: 5px;
}
.portletArchiveListing li.year.expanded span.yearnumber:before {
  content: '▼';
}
.portletArchiveListing ul.years ul.expanded {
  display: block;
}
/* @end */

/* @group Teaser image */
#content .sl-teaser-content-listing .sl-controls .sl-obj-actions > li.document-action-dragme {
  display: none;
}
/* @end */

/* @group tabbed block view */

body.template-tabbed_block_view #content ul.formTabs {
  margin-bottom: 1em;
}
body.template-tabbed_block_view #content #content-core .document-action-dragme {
  display: none !important;
}
body.template-tabbed_block_view #content .simplelayout-content.two-columns-design .twocolumn {
  width: 100%;
  float: none;
}
body.template-tabbed_block_view .slAlignBlocks {
  display: none;
}

/* @end */

/* @group event data table */

div.eventData {
  margin-top: 1em;
}

/* @end */

/* @group colorbox */

#cboxTitle {
  padding-left: 0.5em;
  background: #FFF;
  background: rgba(255, 255, 255, 0.8);
  width: 100%;
  bottom: 20px;
  padding: 0.5em;
  box-sizing: border-box;
  text-align: left;
}

#cboxTitle b{
  display: inline-block;
  width: 100%;
}


/* @end */

/* @group alphabetical subject listing */

#content .alphabetical-subject-listing .letter-index {
  margin: 1em 0em;
}

#content .alphabetical-subject-listing .letter-index span {
  color: #AAA;
}

#content .alphabetical-subject-listing .letter-index a.current {
  color: #000;
}

#content .alphabetical-subject-listing .letter-index li:before {
  content: "|";
  color: #EEE;
}

#content .alphabetical-subject-listing .letter-index li:first-child:before {
  content: "";
}

#content .alphabetical-subject-listing .letter-index li {
  display: inline;
  padding-right: 0.1em;
}

#content .alphabetical-subject-listing .subject-content {
  list-style: none;
  margin-left: 0em;
}

/* @end */

/* @group faq */

.faqtitle {
  background-color: #ddd;
  border-radius: 3px;
}
.faqtitle:hover {
  background-color: #cbcbcb;
}
.faqtitle .itemTitle {
  padding-left: 20px;
  font-size: 1.5em;
  display:block;
}
.faqtitle .itemTitle.imgright {
  background: url(++resource++ftw.contentpage.resources/arrow_right.png) no-repeat left center;
}

.faqtitle .itemTitle.imgdown {
  background: url(++resource++ftw.contentpage.resources/arrow_down.png) no-repeat left center;
}

li.slAction a.layout_dummy-dummy-faq {
  background: url(++resource++ftw.contentpage.resources/faq.png) no-repeat center center;
}

/* @end */
